@@ -15,6 +15,14 @@ if not on_rtd:  # only import and set the theme if we're building docs locally
     html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
 
 
+autodoc_default_flags = [
+    "members"
+    , "show-inheritance"
+    , "inherited-members"
+]
+
+autoclass_content = 'both'
+
 extensions = [
     'sphinx.ext.autodoc',
     'sphinx.ext.intersphinx',
@@ -47,10 +55,10 @@ texinfo_documents = [
      'Miscellaneous'),
 ]
 
-intersphinx_mapping = {'python': ('http://docs.python.org/', None)
+intersphinx_mapping = {
+    'python': ('http://docs.python.org/', None),
+    'rediscluster': ('https://redis-py-cluster.readthedocs.io/en/latest/', None),
+    'redis': ('https://redis-py.readthedocs.io/en/latest/', None),
+    'pymemcache': ('https://pymemcache.readthedocs.io/en/latest/', None)
 }
 
-autodoc_default_flags = [
-    "members"
-    , "show-inheritance"
-]
