@@ -41,13 +41,16 @@ def storage_from_string(storage_string, **options):
 class StorageRegistry(type):
     def __new__(mcs, name, bases, dct):
         storage_scheme = dct.get('STORAGE_SCHEME', None)
-        if not bases == (object, ) and not storage_scheme:
+        if not bases == (object,) and not storage_scheme:
             raise ConfigurationError(
                 "%s is not configured correctly, it must specify a STORAGE_SCHEME class attribute"
                 % name
             )
         cls = super(StorageRegistry, mcs).__new__(mcs, name, bases, dct)
-        SCHEMES[storage_scheme] = cls
+        if storage_scheme:
+            schemes = [storage_scheme] if isinstance(storage_scheme, six.string_types) else storage_scheme
+            for scheme in schemes:
+                SCHEMES[scheme] = cls
         return cls
 
 
@@ -101,6 +104,14 @@ class Storage(object):
         """
         raise NotImplementedError
 
+    @abstractmethod
+    def clear(self, key):
+        """
+        resets the rate limit key
+        :param str key: the key to clear rate limits for
+        """
+        raise NotImplementedError
+
 
 class LockableEntry(threading._RLock):
     __slots__ = ["atime", "expiry"]
@@ -118,7 +129,7 @@ class MemoryStorage(Storage):
     and a simple list to implement moving window strategy.
 
     """
-    STORAGE_SCHEME = "memory"
+    STORAGE_SCHEME = ["memory"]
 
     def __init__(self, uri=None, **_):
         self.storage = Counter()
@@ -172,6 +183,14 @@ class MemoryStorage(Storage):
             self.expirations.pop(key, None)
         return self.storage.get(key, 0)
 
+    def clear(self, key):
+        """
+        :param str key: the key to clear rate limits for
+        """
+        self.storage.pop(key, None)
+        self.expirations.pop(key, None)
+        self.events.pop(key, None)
+
     def acquire_entry(self, key, limit, expiry, no_add=False):
         """
         :param str key: rate limit key to acquire an entry in
@@ -179,7 +198,7 @@ class MemoryStorage(Storage):
         :param int expiry: expiry of the entry
         :param bool no_add: if False an entry is not actually acquired but instead
          serves as a 'check'
-        :return: True/False
+        :rtype: bool
         """
         self.events.setdefault(key, [])
         self.__schedule_expiry()
@@ -219,6 +238,7 @@ class MemoryStorage(Storage):
 
         :param str key: rate limit key
         :param int expiry: expiry of entry
+        :return: (start of window, number of acquired entries)
         """
         timestamp = time.time()
         acquired = self.get_num_acquired(key, expiry)
@@ -313,12 +333,20 @@ class RedisInteractor(object):
         """
         return int(connection.get(key) or 0)
 
+    def clear(self, key, connection):
+        """
+        :param str key: the key to clear rate limits for
+        :param connection: Redis connection
+        """
+        connection.delete(key)
+
     def get_moving_window(self, key, limit, expiry):
         """
         returns the starting point and the number of entries in the moving window
 
         :param str key: rate limit key
         :param int expiry: expiry of entry
+        :return: (start of window, number of acquired entries)
         """
         timestamp = time.time()
         window = self.lua_moving_window([key],
@@ -347,7 +375,7 @@ class RedisInteractor(object):
         :param str key: the key to get the expiry for
         :param connection: Redis connection
         """
-        return int((connection.ttl(key) or 0) + time.time())
+        return int(max(connection.ttl(key), 0) + time.time())
 
     def check(self, connection):
         """
@@ -362,25 +390,34 @@ class RedisInteractor(object):
 
 class RedisStorage(RedisInteractor, Storage):
     """
-    rate limit storage with redis as backend
+    Rate limit storage with redis as backend.
+
+    Depends on the `redis-py` library.
     """
 
-    STORAGE_SCHEME = "redis"
+    STORAGE_SCHEME = ["redis", "rediss", "redis+unix"]
 
-    def __init__(self, uri, **_):
+    def __init__(self, uri, **options):
         """
-        :param str uri: uri of the form 'redis://host:port or redis://host:port/db'
+        :param str uri: uri of the form `redis://[:password]@host:port`,
+         `redis://[:password]@host:port/db`,
+         `rediss://[:password]@host:port`, `redis+unix:///path/to/sock` etc.
+         This uri is passed directly to :func:`redis.from_url` except for the
+         case of `redis+unix` where it is replaced with `unix`.
+        :param \*\*options: all remaining keyword arguments are passed
+         directly to the constructor of :class:`redis.Redis`
         :raise ConfigurationError: when the redis library is not available
         """
         if not get_dependency("redis"):
             raise ConfigurationError(
                 "redis prerequisite not available"
             )  # pragma: no cover
-        self.storage = get_dependency("redis").from_url(uri)
+        uri = uri.replace('redis+unix', 'unix')
+        self.storage = get_dependency("redis").from_url(uri, **options)
         self.initialize_storage(uri)
         super(RedisStorage, self).__init__()
 
-    def initialize_storage(self, uri):
+    def initialize_storage(self, _uri):
         self.lua_moving_window = self.storage.register_script(
             self.SCRIPT_MOVING_WINDOW
         )
@@ -413,6 +450,12 @@ class RedisStorage(RedisInteractor, Storage):
         """
         return super(RedisStorage, self).get(key, self.storage)
 
+    def clear(self, key):
+        """
+        :param str key: the key to clear rate limits for
+        """
+        return super(RedisStorage, self).clear(key, self.storage)
+
     def acquire_entry(self, key, limit, expiry, no_add=False):
         """
         :param str key: rate limit key to acquire an entry in
@@ -439,43 +482,36 @@ class RedisStorage(RedisInteractor, Storage):
         return super(RedisStorage, self).check(self.storage)
 
     def reset(self):
-        """WARNING, this operation was designed to be fast, but was not tested
-        on a large production based system. Be careful with its usage as it
-        could be slow on very large data sets.
-
+        """
         This function calls a Lua Script to delete keys prefixed with 'LIMITER'
-        in block of 5000."""
+        in block of 5000.
+
+        .. warning::
+           This operation was designed to be fast, but was not tested
+           on a large production based system. Be careful with its usage as it
+           could be slow on very large data sets.
+
+        """
 
         cleared = self.lua_clear_keys(['LIMITER*'])
         return cleared
 
 
-class RedisSSLStorage(RedisStorage):
-    """
-    rate limit storage with redis as backend using SSL connection
-    """
-
-    STORAGE_SCHEME = "rediss"
-
-    def __init__(self, uri, **options):
-        """
-        :param str uri: uri of the form 'rediss://host:port or rediss://host:port/db'
-        :raise ConfigurationError: when the redis library is not available
-         or if the redis host cannot be pinged.
-        """
-        super(RedisSSLStorage, self).__init__(uri, **options)  #noqa
-
-
 class RedisSentinelStorage(RedisStorage):
     """
-    rate limit storage with redis sentinel as backend
+    Rate limit storage with redis sentinel as backend
+
+    Depends on `redis-py` library
     """
 
-    STORAGE_SCHEME = "redis+sentinel"
+    STORAGE_SCHEME = ["redis+sentinel"]
 
-    def __init__(self, uri, **options):
+    def __init__(self, uri, service_name=None, **options):
         """
-        :param str uri: url of the form 'redis+sentinel://host:port,host:port/service_name'
+        :param str uri: url of the form `redis+sentinel://host:port,host:port/service_name`
+        :param str service_name, optional: sentinel service name (if not provided in `uri`)
+        :param \*\*options: all remaining keyword arguments are passed
+         directly to the constructor of :class:`redis.sentinel.Sentinel`
         :raise ConfigurationError: when the redis library is not available
          or if the redis master host cannot be pinged.
         """
@@ -494,32 +530,30 @@ class RedisSentinelStorage(RedisStorage):
             sentinel_configuration.append((host, int(port)))
         self.service_name = (
             parsed.path.replace("/", "")
-            if parsed.path else options.get("service_name", None)
+            if parsed.path else service_name
         )
         if self.service_name is None:
             raise ConfigurationError("'service_name' not provided")
+
+        options.setdefault('socket_timeout', 0.2)
+
         self.sentinel = get_dependency("redis.sentinel").Sentinel(
             sentinel_configuration,
-            socket_timeout=options.get("socket_timeout", 0.2),
-            password=password
+            password=password,
+            **options
         )
+        self.storage = self.sentinel.master_for(self.service_name)
+        self.storage_slave = self.sentinel.slave_for(self.service_name)
         self.initialize_storage(uri)
         super(RedisStorage, self).__init__()
 
-    @property
-    def storage(self):
-        return self.sentinel.master_for(self.service_name)
-
-    @property
-    def storage_slave(self):
-        return self.sentinel.slave_for(self.service_name)
-
     def get(self, key):
         """
         :param str key: the key to get the counter value for
         """
         return super(RedisStorage, self).get(key, self.storage_slave)
 
+
     def get_expiry(self, key):
         """
         :param str key: the key to get the expiry for
@@ -535,13 +569,17 @@ class RedisSentinelStorage(RedisStorage):
 
 class RedisClusterStorage(RedisStorage):
     """
-    rate limit storage with redis cluster as backend
+    Rate limit storage with redis cluster as backend
+
+    Depends on `redis-py-cluster` library
     """
-    STORAGE_SCHEME = "redis+cluster"
+    STORAGE_SCHEME = ["redis+cluster"]
 
     def __init__(self, uri, **options):
         """
-        :param str uri: url of the form 'redis+cluster://host:port,host:port'
+        :param str uri: url of the form `redis+cluster://[:password]@host:port,host:port`
+        :param \*\*options: all remaining keyword arguments are passed
+         directly to the constructor of :class:`rediscluster.RedisCluster`
         :raise ConfigurationError: when the rediscluster library is not available
          or if the redis host cannot be pinged.
         """
@@ -554,9 +592,12 @@ class RedisClusterStorage(RedisStorage):
         for loc in parsed.netloc.split(","):
             host, port = loc.split(":")
             cluster_hosts.append({"host": host, "port": int(port)})
+
+        options.setdefault('max_connections', 1000)
+
         self.storage = get_dependency("rediscluster").RedisCluster(
             startup_nodes=cluster_hosts,
-            max_connections=options.get("max_connections", 1000)
+            **options
         )
         self.initialize_storage(uri)
         super(RedisStorage, self).__init__()
@@ -568,9 +609,10 @@ class RedisClusterStorage(RedisStorage):
         keys that are prefixed with 'LIMITER' and calls delete on them, one at
         a time.
 
-        WARNING, this operation was not tested with extremely large data sets.
-        On a large production based system, care should be taken with its
-        usage as it could be slow on very large data sets"""
+        .. warning::
+         This operation was not tested with extremely large data sets.
+         On a large production based system, care should be taken with its
+         usage as it could be slow on very large data sets"""
 
         keys = self.storage.keys('LIMITER*')
         return sum([self.storage.delete(k.decode('utf-8')) for k in keys])
@@ -578,26 +620,38 @@ class RedisClusterStorage(RedisStorage):
 
 class MemcachedStorage(Storage):
     """
-    rate limit storage with memcached as backend
+    Rate limit storage with memcached as backend.
+
+    Depends on the `pymemcache` library.
     """
     MAX_CAS_RETRIES = 10
-    STORAGE_SCHEME = "memcached"
+    STORAGE_SCHEME = ["memcached"]
 
     def __init__(self, uri, **options):
         """
         :param str uri: memcached location of the form
-         'memcached://host:port,host:port'
-        :raise ConfigurationError: when pymemcached is not available
+         `memcached://host:port,host:port`, `memcached:///var/tmp/path/to/sock`
+        :param \*\*options: all remaining keyword arguments are passed
+         directly to the constructor of :class:`pymemcache.client.base.Client`
+        :raise ConfigurationError: when `pymemcache` is not available or memcached
+         location cannot be parsed.
         """
         parsed = urllib.parse.urlparse(uri)
-        self.cluster = []
+        self.hosts = []
         for loc in parsed.netloc.strip().split(","):
             if not loc:
                 continue
             host, port = loc.split(":")
-            self.cluster.append((host, int(port)))
-        self.library = options.get('library', 'pymemcache.client')
-        self.client_getter = options.get('client_getter', self.get_client)
+            self.hosts.append((host, int(port)))
+        else:
+            # filesystem path to UDS
+            if parsed.path and not parsed.netloc and not parsed.port:
+                self.hosts = [parsed.path]
+
+        self.library = options.pop('library', 'pymemcache.client')
+        self.cluster_library = options.pop('library', 'pymemcache.client.hash')
+        self.client_getter = options.pop('client_getter', self.get_client)
+        self.options = options
 
         if not get_dependency(self.library):
             raise ConfigurationError(
@@ -607,18 +661,22 @@ class MemcachedStorage(Storage):
         self.local_storage = threading.local()
         self.local_storage.storage = None
 
-    def get_client(self, module, hosts):
+    def get_client(self, module, hosts, **kwargs):
         """
         returns a memcached client.
         :param module: the memcached module
         :param hosts: list of memcached hosts
         :return:
         """
-        return module.Client(*hosts)
+        return (
+            module.HashClient(hosts, **kwargs)
+            if len(hosts) > 1 else module.Client(*hosts, **kwargs)
+        )
 
     def call_memcached_func(self, func, *args, **kwargs):
         if 'noreply' in kwargs:
-            if 'noreply' not in inspect.getargspec(func).args:
+            argspec = inspect.getargspec(func)
+            if not ('noreply' in argspec.args or argspec.keywords):
                 kwargs.pop('noreply')  # noqa
         return func(*args, **kwargs)
 
@@ -628,20 +686,29 @@ class MemcachedStorage(Storage):
         lazily creates a memcached client instance using a thread local
         """
         if not (
-            hasattr(self.local_storage, "storage")
-            and self.local_storage.storage
+                hasattr(self.local_storage, "storage")
+                and self.local_storage.storage
         ):
             self.local_storage.storage = self.client_getter(
-                get_dependency(self.library), self.cluster
+                get_dependency(self.cluster_library if len(self.hosts) > 1 else self.library),
+                self.hosts, **self.options
             )
+
         return self.local_storage.storage
 
+
     def get(self, key):
         """
         :param str key: the key to get the counter value for
         """
         return int(self.storage.get(key) or 0)
 
+    def clear(self, key):
+        """
+        :param str key: the key to clear rate limits for
+        """
+        self.storage.delete(key)
+
     def incr(self, key, expiry, elastic_expiry=False):
         """
         increments the counter for a given rate limit key
@@ -695,7 +762,7 @@ class MemcachedStorage(Storage):
         check if storage is healthy
         """
         try:
-            self.call_memcached_func(self.storage.stats)
+            self.call_memcached_func(self.storage.get, 'limiter-check')
             return True
         except:  # noqa
             return False
@@ -706,7 +773,7 @@ class GAEMemcachedStorage(MemcachedStorage):
     rate limit storage with GAE memcache as backend
     """
     MAX_CAS_RETRIES = 10
-    STORAGE_SCHEME = "gaememcached"
+    STORAGE_SCHEME = ["gaememcached"]
 
     def __init__(self, uri, **options):
         options["library"] = "google.appengine.api.memcache"
