@@ -50,6 +50,8 @@ class RateLimiter(object):
         """
         raise NotImplementedError
 
+    def clear(self, item, *identifiers):
+        return self.storage().clear(item.key_for(*identifiers))
 
 class MovingWindowRateLimiter(RateLimiter):
     """
