@@ -4,9 +4,15 @@ from limits import limits
 
 class GranularityTests(unittest.TestCase):
     def test_seconds_value(self):
-        self.assertEqual(limits.RateLimitItemPerHour(1).get_expiry(), 60 * 60)
-        self.assertEqual(limits.RateLimitItemPerMinute(1).get_expiry(), 60)
-        self.assertEqual(limits.RateLimitItemPerSecond(1).get_expiry(), 1)
+        self.assertEqual(
+            limits.RateLimitItemPerSecond(1).get_expiry(), 1
+        )
+        self.assertEqual(
+            limits.RateLimitItemPerMinute(1).get_expiry(), 60
+        )
+        self.assertEqual(
+            limits.RateLimitItemPerHour(1).get_expiry(), 60 * 60
+        )
         self.assertEqual(
             limits.RateLimitItemPerDay(1).get_expiry(), 60 * 60 * 24
         )
@@ -18,18 +24,24 @@ class GranularityTests(unittest.TestCase):
         )
 
     def test_representation(self):
-        self.assertTrue("1 per 1 hour" in str(limits.RateLimitItemPerHour(1)))
+        self.assertTrue(
+            "1 per 1 second" in str(limits.RateLimitItemPerSecond(1))
+        )
         self.assertTrue(
             "1 per 1 minute" in str(limits.RateLimitItemPerMinute(1))
         )
         self.assertTrue(
-            "1 per 1 second" in str(limits.RateLimitItemPerSecond(1))
+            "1 per 1 hour" in str(limits.RateLimitItemPerHour(1))
+        )
+        self.assertTrue(
+            "1 per 1 day" in str(limits.RateLimitItemPerDay(1))
         )
-        self.assertTrue("1 per 1 day" in str(limits.RateLimitItemPerDay(1)))
         self.assertTrue(
             "1 per 1 month" in str(limits.RateLimitItemPerMonth(1))
         )
-        self.assertTrue("1 per 1 year" in str(limits.RateLimitItemPerYear(1)))
+        self.assertTrue(
+            "1 per 1 year" in str(limits.RateLimitItemPerYear(1))
+        )
 
     def test_comparison(self):
         self.assertTrue(
