@@ -1,6 +1,3 @@
-"""
-
-"""
 import threading
 import time
 import unittest
@@ -24,8 +21,9 @@ from tests import skip_if_pypy
 
 class WindowTests(unittest.TestCase):
     def setUp(self):
-        pymemcache.client.Client(('localhost', 11211)).flush_all()
-        redis.Redis().flushall()
+        pymemcache.client.Client(('localhost', 22122)).flush_all()
+        redis.from_url("redis://localhost:7379").flushall()
+        redis.from_url("redis://:sekret@localhost:7389").flushall()
         redis.sentinel.Sentinel([
             ("localhost", 26379)
         ]).master_for("localhost-redis-sentinel").flushall()
@@ -67,7 +65,7 @@ class WindowTests(unittest.TestCase):
             self.assertEqual(limiter.get_window_stats(limit)[0], start + 2)
 
     def test_fixed_window_with_elastic_expiry_memcache(self):
-        storage = MemcachedStorage('memcached://localhost:11211')
+        storage = MemcachedStorage('memcached://localhost:22122')
         limiter = FixedWindowElasticExpiryRateLimiter(storage)
         limit = RateLimitItemPerSecond(10, 2)
         self.assertTrue(all([limiter.hit(limit) for _ in range(0, 10)]))
@@ -77,26 +75,26 @@ class WindowTests(unittest.TestCase):
         self.assertFalse(limiter.hit(limit))
 
     def test_fixed_window_with_elastic_expiry_memcache_concurrency(self):
-        storage = MemcachedStorage('memcached://localhost:11211')
+        storage = MemcachedStorage('memcached://localhost:22122')
         limiter = FixedWindowElasticExpiryRateLimiter(storage)
         start = int(time.time())
-        limit = RateLimitItemPerSecond(100, 2)
+        limit = RateLimitItemPerSecond(10, 2)
 
         def _c():
-            for i in range(0, 50):
+            for i in range(0, 5):
                 limiter.hit(limit)
 
         t1, t2 = threading.Thread(target=_c), threading.Thread(target=_c)
         t1.start(), t2.start()
-        [t1.join(), t2.join()]
+        t1.join(), t2.join()
         self.assertEqual(limiter.get_window_stats(limit)[1], 0)
         self.assertTrue(
             start + 2 <= limiter.get_window_stats(limit)[0] <= start + 3
         )
-        self.assertEqual(storage.get(limit.key_for()), 100)
+        self.assertEqual(storage.get(limit.key_for()), 10)
 
     def test_fixed_window_with_elastic_expiry_redis(self):
-        storage = RedisStorage('redis://localhost:6379')
+        storage = RedisStorage('redis://localhost:7379')
         limiter = FixedWindowElasticExpiryRateLimiter(storage)
         limit = RateLimitItemPerSecond(10, 2)
         self.assertTrue(all([limiter.hit(limit) for _ in range(0, 10)]))
@@ -144,7 +142,7 @@ class WindowTests(unittest.TestCase):
 
     @skip_if_pypy
     def test_moving_window_redis(self):
-        storage = RedisStorage("redis://localhost:6379")
+        storage = RedisStorage("redis://localhost:7379")
         limiter = MovingWindowRateLimiter(storage)
         limit = RateLimitItemPerSecond(10, 2)
         for i in range(0, 10):
@@ -158,7 +156,7 @@ class WindowTests(unittest.TestCase):
         self.assertEqual(limiter.get_window_stats(limit)[1], 0)
 
     def test_moving_window_memcached(self):
-        storage = MemcachedStorage('memcacheD://localhost:11211')
+        storage = MemcachedStorage('memcached://localhost:22122')
         self.assertRaises(
             NotImplementedError, MovingWindowRateLimiter, storage
         )
